'use strict';

exports.__esModule = true;
exports.default = getStoredState;

var _constants = require('./constants');

function getStoredState(config, onComplete) {
  var transforms = config.transforms || [];
  var storageKey = '' + (config.keyPrefix !== undefined ? config.keyPrefix : _constants.KEY_PREFIX) + config.key;

  // storage with keys -> getAllKeys for localForage support
  var storage = config.storage;

  var restoredState = {};
  var completionCount = 0;

  storage.getItem(storageKey, function (err, serialized) {
    if (err) {
      if (process.env.NODE_ENV !== 'production') console.warn('redux-p/getStoredState: Error in storage.get');
      onComplete(err);
    }

    if (!serialized) onComplete(null, undefined);else {
      try {
        var state = {};
        var rawState = deserializer(serialized);
        Object.keys(rawState).forEach(function (key) {
          state[key] = transforms.reduceRight(function (subState, transformer) {
            return transformer.out(subState, key);
          }, deserializer(rawState[key]));
        });
        onComplete(null, state);
      } catch (err) {
        if (process.env.NODE_ENV !== 'production') console.error('redux-persist/getStoredState: Error restoring data ' + serialized, err);
        onComplete(err);
      }
    }
  });

  if (typeof onComplete !== 'function' && !!Promise) {
    return new Promise(function (resolve, reject) {
      onComplete = function onComplete(err, restoredState) {
        if (err) reject(err);else resolve(restoredState);
      };
    });
  }
}

function deserializer(serial) {
  return JSON.parse(serial);
}