'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.default = persistReducer;

var _constants = require('./constants');

var _stateReconciler = require('./stateReconciler');

var _stateReconciler2 = _interopRequireDefault(_stateReconciler);

var _createPersistoid = require('./createPersistoid');

var _createPersistoid2 = _interopRequireDefault(_createPersistoid);

var _getStoredState = require('./getStoredState');

var _getStoredState2 = _interopRequireDefault(_getStoredState);

var _purgeStoredState = require('./purgeStoredState');

var _purgeStoredState2 = _interopRequireDefault(_purgeStoredState);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

/*
  @TODO add validation / handling for:
  - persisting a reducer which has nested _persist
  - handling actions that fire before reydrate is called
*/
function persistReducer(config, baseReducer) {
  if (process.env.NODE_ENV !== 'production') {
    if (!config) throw new Error('config is required for persistReducer');
    if (!config.key) throw new Error('key is required in persistor config');
    if (!config.storage) throw new Error("redux-persist: config.storage is required. Try using `import storageLocal from 'redux-persist/es/storages/local'");
  }

  var version = config.version || _constants.DEFAULT_VERSION;
  var debug = config.debug || false;
  var _persistoid = null;
  var _purge = false;

  // $FlowFixMe perhaps there is a better way to do this?
  var defaultState = baseReducer(undefined, {
    type: 'redux-persist/default-probe'
  });
  if (process.env.NODE_ENV !== 'production') {
    if (Array.isArray(defaultState) || (typeof defaultState === 'undefined' ? 'undefined' : _typeof(defaultState)) !== 'object') console.error('redux-persist: does not yet support non plain object state.');
  }
  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultState;
    var action = arguments[1];

    var _ref = state || {},
        _persist = _ref._persist,
        rest = _objectWithoutProperties(_ref, ['_persist']);

    var restState = rest;

    if (action.type === _constants.PERSIST) {
      if (state._persist) {
        // @NOTE PERSIST can be called multiple times, noop after the first
        return state;
      }
      if (typeof action.rehydrate !== 'function') throw new Error('redux-persist: action.rehydrate is not a function. This can happen if the action is being replayed. This is an unexplored use case, please open an issue and we will figure out a resolution.');
      if (typeof action.register !== 'function') throw new Error('redux-persist: action.register is not a function. This can happen if the action is being replayed. This is an unexplored use case, please open an issue and we will figure out a resolution.');

      var rehydrate = action.rehydrate;
      action.register(config.key);

      (0, _getStoredState2.default)(config, function (err, restoredState) {
        _persistoid = (0, _createPersistoid2.default)(config);
        if (err) {
          action.rehydrate(config.key, undefined, err);
        } else {
          var migrate = config.migrate || function (s, v) {
            return Promise.resolve(s);
          };
          migrate(restoredState, version).then(function (migratedState) {
            action.rehydrate(config.key, migratedState);
          }, function (migrateErr) {
            if (process.env.NODE_ENV !== 'production' && migrateErr) console.error('redux-persist: migration error', migrateErr);
            action.rehydrate(config.key, undefined, migrateErr);
          });
        }
      });

      return _extends({}, state, { _persist: { version: version, rehydrated: false } });
    } else if (action.type === _constants.PURGE) {
      _purge = true;
      (0, _purgeStoredState2.default)(config);
      return state;
    } else if (action.type === _constants.REHYDRATE) {
      // noop if purging
      if (_purge) return state;

      // @NOTE if key does not match, will continue to default else below
      if (action.key === config.key) {
        var reducedState = baseReducer(restState, action);
        var inboundState = action.payload;
        var reconciledRest = (0, _stateReconciler2.default)(state, inboundState, reducedState, config);

        return _extends({}, reconciledRest, {
          _persist: _extends({}, _persist, { rehydrated: true })
        });
      }
    }

    // @NOTE default pass through reducer if the switch statement above did not return
    // @TODO more performant workaround for combineReducers warning
    var newState = _extends({}, baseReducer(restState, action), {
      _persist: _persist
    });
    _persistoid && _persistoid.update(newState);
    return newState;
  };
}